/**
 * Mixin for scope
 * @function scopeMixin
 */
'use strict'

const abind = require('abind')

/** @lends scopeMixin */
function scopeMixin (BaseClass) {
  return class ScopeMixed extends BaseClass {
    constructor (...args) {
      super(...args)
      const s = this
      abind(s, { proto: ScopeMixed.prototype })
    }

    scopeValue (...keys) {
      const s = this
      s.assertScopeName()
      let { scopeName } = s
      return s.get(scopeName, ...keys)
    }

    scopeOperate (operationType, fieldName, value, isError = false) {
      const s = this
      s.assertScopeName()
      let { scopeName } = s
      let actionType = `${scopeName}:${operationType}`
      return s.dispatch(actionType, fieldName, value, isError)
    }

    assertScopeName () {
      const s = this
      let { scopeName } = s
      if (!scopeName) {
        throw new Error('[ScopeMixin] scopeName is missing!')
      }
    }
  }
}

module.exports = scopeMixin
